/*
 * FFT.c
 *
 *  Created on: Oct 10, 2011
 *      Author: GSI
 */

#include <Dsplib.h>
#include <usbstk5515.h>
#include <AIC_func.h>
#include <usbstk5515_interrupts.h>
#include <stdio.h>
#include <math.h>

#define BUFSIZE 512
#define BUFCHK	2
#define N_SINE 256
#define FTV 1365
#define PI 3.14159265
Uint32 Counter;

// We need three buffers.
// One buffer will be used by the AIC to dump data into (pAIC).  Once that
// buffer is full, that buffer will be used as the input to the FFT.  A third
// buffer is used for the output of the FFT.
DATA bOne	[2*BUFSIZE];//Buffer 1
DATA bTwo 	[2*BUFSIZE];//Buffer 2
DATA bOut	[2*BUFSIZE];//Buffer 3--FFT output goes here.
Int16 sineTable [N_SINE];

DATA *pAIC, *pFFT, *pDONE;//Pointers to Current buffers

DATA AMP;
void Reset();


//Determine square root for Q number.  Takes a 32-bit Q_FRACBITS and returns
//number in the same representation. Taken from:
// http://jet.ro/files/The_neglected_art_of_Fixed_Point_arithmetic_20060913.pdf
long FFracSqrt(long x)
{
	/* The definitions below yield 2 integer bits, 30 fractional bits */
	#define FRACBITS 30    /* Must be even! */
	#define ITERS    (15 + (FRACBITS >> 1))

    unsigned long root, remHi, remLo, testDiv, count;
    root = 0;         /* Clear root */
    remHi = 0;        /* Clear high part of partial remainder */
    remLo = x;        /* Get argument into low part of partial remainder */
    count = ITERS;    /* Load loop counter */
    do {
        remHi = (remHi << 2) | (remLo >> 30); remLo <<= 2;  /* get 2 bits of arg */
        root <<= 1;   /* Get ready for the next bit in the root */
        testDiv = (root << 1) + 1;    /* Test radical */
        if (remHi >= testDiv) {
            remHi -= testDiv;
            root += 1;
        }
    } while (count-- != 0);
return(root);
}

Uint16 Acc = 0;
Int16 DDS(Uint16 ftv)
{
	Uint16 index;
	index = Acc>>8; // using a 256 value table
	Acc += ftv;
	return sineTable[index];
}

interrupt void I2S_ISR()
{
	Int16 right, left,output;
	AIC_read2(&right, &left);
	output = ((Int32)AMP * (Int32)DDS(FTV) )>>15;
	AIC_write2(output, right);
	pAIC[Counter] = right;  //Only use evens for FFT function
	pAIC[Counter+1] = 0;	//No imaginary part
	Counter += 2;
	IFR0 &= (1 << I2S_BIT_POS);//Clear interrupt Flag
}



// Setup AIC interrupt routine.
void I2S_interrupt_setup(void)
{
	Uint32 reset_loc = (Uint32)Reset;
	IVPD = reset_loc >> 8;//pointer table points to Reset
	IVPH = reset_loc >> 8;//Ditto

	*((Uint32*)((reset_loc + I2S_ISR_OFFSET)>>1)) = (Uint32)I2S_ISR;

	SYS_EXBUSSEL |= (0x1 << 8);//Set the External bus select SP0Mode to I2S
	IER0 |= (1 << I2S_BIT_POS);//Set up the Global interrupt register to flag on I2S receive flag
	IFR0 &= (1 << I2S_BIT_POS);
}
void main(void)
{
	Int32 tmp,tmp2;
	Int16 ctr;
	DATA *temp1;
	pAIC = &bOne[0];
	pFFT = &bTwo[0];
	pDONE = &bOut[0];
	Counter = AMP = 0;

	for(ctr=0; ctr<N_SINE; ctr++)  // generate sine table
		sineTable[ctr]=(Int32)(32767.0*sin(2.0*PI*ctr/N_SINE));

	USBSTK5515_init();
	AIC_init();
	I2S_interrupt_setup();
	_enable_interrupts();

	while(1)
	{
		if(Counter >= (BUFSIZE*2))
		{
			//Rotate buffers
			temp1= pFFT;
			pFFT = pAIC;
			pAIC = temp1;

			// reset Counter
			Counter = 0;

			//Do cfft with scaling.
			cfft_SCALE(pFFT, BUFSIZE);
			cbrev(pFFT, pDONE, BUFSIZE);

			//Find magnitude of X(BUFCHK/2).
			tmp = ((Int32)pDONE[BUFCHK]*pDONE[BUFCHK]) + ((Int32)pDONE[BUFCHK+1] * pDONE[BUFCHK+1]);
			tmp2 = FFracSqrt(tmp);
			tmp2 = tmp2 >> 14;  // Should really be >>15.  Done for larger dynamic
			                     // range.  But now we need to check for overflow.
			AMP = tmp2>32767?32767:tmp2<-32768?-32768:tmp2; // amplitude of the output sine wave

			if(Counter >= (BUFSIZE*2))  // If this happens, we are too slow!
				goto TERMINATE;
		}
	}
TERMINATE:
	printf("Not enough time to compute FFT\n");
}

